@@ -6,14 +6,14 @@ data.
 .. include:: ./documentation.md
 """
 from .assistant import Assistant, AssistantContext, AssistantResponse
+from .cloud import KodexaPlatform, RemoteAction, RemotePipeline, RemoteSession
 from .connectors import FileHandleConnector, FolderConnector, UrlConnector, add_connector, get_connector, \
     get_connectors, get_source, registered_connectors
 from .model import ContentEvent, ContentFeature, ContentNode, Document, DocumentActor, DocumentFamily, DocumentMetadata, \
-    DocumentStore, DocumentTransition, SourceMetadata, TransitionType
-from .model.objects import Taxonomy
+    DocumentStore, DocumentTransition, RemoteStore, SourceMetadata, TransitionType
 from .pipeline import Pipeline, PipelineContext, PipelineStatistics
-from .platform import KodexaPlatform, RemoteAction, RemotePipeline, RemoteSession
 from .steps import NodeTagCopy, NodeTagger, RollupTransformer, TagsToKeyValuePairExtractor, TextParser, \
     KodexaProcessingException
-from .stores import LocalDocumentStore, LocalModelStore, RemoteDocumentStore, \
-    RemoteModelStore, RemoteDataStore, TableDataStore
+from .stores import DataStoreHelper, LocalDocumentStore, LocalModelStore, RemoteDocumentStore, \
+    RemoteModelStore, RemoteTableDataStore, TableDataStore
+from .taxonomy import RemoteTaxonomy, Taxon, Taxonomy
