@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import inspect
 import logging
+import os
 import sys
 import time
 import traceback
@@ -112,8 +113,7 @@ class PipelineContext:
 
     def __init__(self, content_provider=None, store_provider=None,
                  existing_content_objects=None,
-                 context=None, execution_id=None,
-                 status_handler=None, cancellation_handler=None):
+                 context=None, execution_id=None):
         if content_provider is None:
             content_provider = InMemoryContentProvider()
         if store_provider is None:
@@ -135,18 +135,6 @@ class PipelineContext:
         self.document_family = None
         self.content_object = None
         self.document_store = None
-        self.status_handler = status_handler
-        self.cancellation_handler = cancellation_handler
-
-    def update_status(self, status_message: str, status_full_message: Optional[str] = None):
-        if self.status_handler is not None:
-            self.status_handler(status_message, status_full_message)
-
-    def is_cancelled(self) -> bool:
-        if self.cancellation_handler is not None:
-            return self.cancellation_handler()
-        else:
-            return False
 
     def get_context(self) -> Dict:
         """ """
@@ -417,6 +405,13 @@ class PipelineStep:
         else:
             return document
 
+    def end_processing(self, context):
+        try:
+            self.step.end_processing(context)
+        except:
+            pass
+
+
 class LabelStep(object):
     """A simple step for handling the labelling for a document"""
 
@@ -424,6 +419,10 @@ class LabelStep(object):
         self.label = label
         self.remove = remove
 
+    def get_name(self):
+        """ """
+        return f"Remove label {self.label}" if self.remove else f"Add label {self.label}"
+
     def process(self, document: Document):
         """
 
@@ -702,6 +701,11 @@ class Pipeline:
             else:
                 logger.warning("A step did not return a document?")
 
+        logger.info(f"Completing pipeline {self.name}")
+
+        for step in self.steps:
+            step.end_processing(self.context)
+
         logger.info(f"Completed pipeline {self.name}")
 
         return self.context
