@@ -2,4 +2,4 @@
 Common and reusable steps
 """
 from .common import NodeTagger, NodeTagCopy, TextParser, RollupTransformer, TagsToKeyValuePairExtractor, \
-    KodexaProcessingException
+    KodexaProcessingException, DocumentStoreWriter
